import json
import typer
from . import rest_helper as rest

app = typer.Typer()


@app.command(name="list", help="List your projects")
def list_projects(organisation: str = typer.Option(..., help="Organisation ID", envvar='REMOTIVE_CLOUD_ORGANISATION')):
    r = rest.handle_get(url=f'/api/bu/{organisation}/me',  return_response=True)
    if r.status_code == 200:
        # extract the project uid parts
        projects = r.json()['projects']
        projects = map(lambda p: p['uid'], projects)
        print(json.dumps(list(projects)))
    else:
        print(r.status_code)
        print(r.text)


@app.command(name="create")
def create_project(
    organisation: str = typer.Option(..., help="Organisation ID", envvar='REMOTIVE_CLOUD_ORGANISATION'),
    project_uid: str = typer.Option(..., help="Project UID"),
    project_display_name: str = typer.Option(default="", help="Project display name")
):

    create_project_req = {
        'uid': project_uid,
        'displayName': project_display_name if project_display_name != "" else project_uid,
        'description': ''
    }

    rest.handle_post(url=f'/api/bu/{organisation}/project', body=json.dumps(create_project_req))


@app.command(name="delete")
def delete(project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    rest.handle_delete(url=f'/api/project/{project}')


import json
import os
import sys
import time
import webbrowser
from http.server import BaseHTTPRequestHandler, HTTPServer
from pathlib import Path
from threading import Thread
import typer
from . import rest_helper as rest
from . import auth_tokens


help="""
Manage how you authenticate with our cloud platform
"""

app = typer.Typer(help=help)

app.add_typer(auth_tokens.app, name="tokens", help="Manage users personal access tokens")
config_dir_name = str(Path.home()) + "/.config/.remotive/"
token_file_name = str(Path.home()) + "/.config/.remotive/cloud.secret.token"


class S(BaseHTTPRequestHandler):
    def _set_response(self):
        self.send_response(200)
        # self.send_response(301)
        # self.send_header('Location', 'https://cloud.remotivelabs.com')
        # self.end_headers()
        self.send_header('Content-type', 'text/html')
        self.end_headers()

    def log_message(self, format, *args):
        return

    def do_GET(self):
        self._set_response()
        self.wfile.write("Successfully setup CLI, return to your terminal to continue".encode('utf-8'))
        path = self.path
        time.sleep(1)
        httpd.server_close()

        killerthread = Thread(target=httpd.shutdown)
        killerthread.start()

        if not os.path.exists(config_dir_name):
            os.makedirs(config_dir_name)
        write_token(path[1:])
        print("Successfully logged on, you are ready to go with cli")


def start_local_webserver(server_class=HTTPServer, handler_class=S, port=0):
    server_address = ('', port)
    global httpd
    httpd = server_class(server_address, handler_class)


#
# CLI commands go here
#

@app.command(name="login")
def login():
    """
    Login to the cli using browser

    This will be used as the current access token in all subsequent requests. This would
    be the same as activating a personal access key or service-account access key.
    """
    start_local_webserver()
    webbrowser.open(f'{rest.base_url}/login?redirectUrl=http://localhost:{httpd.server_address[1]}', new=1, autoraise=True)
    httpd.serve_forever()

@app.command()
def whoami():
    """
    Validates authentication and fetches your user information
    """
    rest.handle_get('/api/whoami')

@app.command()
def print_access_token():
    """
    Print current active access token
    """
    print(read_token())








#@app.command(help="Clear access token")
#def logout():
#    write_token("")
#    print("Access token removed")


#@app.command(help="Test authentication")
#def test():
#    if rest.has_access('/api/whoami'):
#        print("Access granted, good to go!")
#    else:
#        print("Access failed (401), please login again")


def read_token():
    f = open(token_file_name, "r")
    token = f.read()
    f.close()
    return token


def read_file_with_path(file):
    f = open(file, "r")
    token = f.read()
    f.close()
    return token


def read_file(file):
    f = open(str(Path.home()) + f"/.config/.remotive/{file}", "r")
    token = f.read()
    f.close()
    return token


def write_token(token):
    f = open(token_file_name, "w")
    f.write(token)
    f.close()




# Key stuff
# f = open(str(Path.home())+ "/.remotivelabs/privatekey.json", "r")
# j = json.loads(f.read())
# print(j['privateKey'])
# key = load_pem_private_key(bytes(j['privateKey'],'UTF-8'), None)
# print(key.key_size)
#
# "exp": datetime.now(tz=timezone.utc)
# encoded = jwt.encode({"some": "payload"}, j['privateKey'] , algorithm="RS256", headers={"kid":  j["keyId"]})
# print(encoded)



import sys

import requests
import os
import json
import logging
from pathlib import Path
from os.path import exists
#from rich import print
import typer

if 'REMOTIVE_CLOUD_HTTP_LOGGING' in os.environ:
    logging.basicConfig()
    logging.getLogger().setLevel(logging.DEBUG)
    requests_log = logging.getLogger("requests.packages.urllib3")
    requests_log.setLevel(logging.DEBUG)
    requests_log.propagate = True
global baseurl
base_url = "https://cloud.remotivelabs.com"

if 'REMOTIVE_CLOUD_BASE_URL' in os.environ:
    base_url = os.environ['REMOTIVE_CLOUD_BASE_URL']

# if 'REMOTIVE_CLOUD_AUTH_TOKEN' not in os.environ:
#    print('export REMOTIVE_CLOUD_AUTH_TOKEN=auth must be set')
#    exit(0)

# token = os.environ["REMOTIVE_CLOUD_AUTH_TOKEN"]
# headers = {"Authorization": "Bearer " + token}

headers = {}
org = ""


def ensure_auth_token():

    #if 'REMOTIVE_CLOUD_ORGANISATION' not in os.environ:
    #    print('You must first set the organisation id to use: export REMOTIVE_CLOUD_ORGANISATION=organisationUid')
    #    raise typer.Exit()
    global org
    #org = os.environ["REMOTIVE_CLOUD_ORGANISATION"]

    if not exists(str(Path.home()) + "/.config/.remotive/cloud.secret.token"):
        print("Access token not found, please login first")
        raise typer.Exit()

    f = open(str(Path.home()) + "/.config/.remotive/cloud.secret.token", "r")
    token = f.read()
    os.environ['REMOTIVE_CLOUD_AUTH_TOKEN'] = token
    global headers
    headers = {
        'Authorization': "Bearer " + token.strip(),
        'User-Agent': 'RemotiveCli 0.0.1alphax',
    }


def handle_get(url, params={},return_response: bool = False):
    ensure_auth_token()
    r = requests.get(f'{base_url}{url}', headers=headers, params=params)

    if return_response:
        return r

    if r.status_code == 200:
        print(json.dumps(r.json()))
    else:
        sys.stderr.write(f'Got status code: {r.status_code}\n')
        sys.stderr.write(r.text + "\n")


def has_access(url, params={}):
    ensure_auth_token()
    r = requests.get(f'{base_url}{url}', headers=headers, params=params)
    if r.status_code == 401:
        return False
    else:
        return True


def handle_delete(url, params={}, quiet=False, success_msg="Successfully deleted"):
    ensure_auth_token()
    r = requests.delete(f'{base_url}{url}', headers=headers, params=params)
    if r.status_code == 200:
        if quiet == False:
            print(json.dumps(r.json()))
    if r.status_code == 204:
        if quiet == False:
            sys.stderr.write(f'{success_msg}\n')
    else:
        print(f'Got status code: {r.status_code}')
        print(r.text)
        typer.Exit(1)

def handle_post(url, body=None, params={}, return_response: bool = False):
    ensure_auth_token()
    headers["content-type"] = "application/json"
    r = requests.post(f'{base_url}{url}', headers=headers, params=params, data=body)

    if return_response:
        return r

    if r.status_code == 200:
        print(r.text)
        # print(json.dumps(r.json()))
    elif r.status_code == 204:
        print(r.status_code)
    else:
        print(f'Got status code: {r.status_code}')
        print(r.text)

def handle_put(url, body=None, params={}, return_response: bool = False):
    ensure_auth_token()
    headers["content-type"] = "application/json"
    r = requests.put(f'{base_url}{url}', headers=headers, params=params, data=body)

    if return_response:
        return r

    if r.status_code == 200:
        print(r.text)
        # print(json.dumps(r.json()))
    elif r.status_code == 204:
        print(r.status_code)
    else:
        print(f'Got status code: {r.status_code}')
        print(r.text)



import json
import signal as os_signal
import os
from time import sleep
from typing import List
from rich import print as rich_rprint
from rich.progress import Progress, SpinnerColumn, TextColumn
import grpc
import typer
from .lib.broker import Broker

app = typer.Typer(help=help)


@app.command(help="List signals names on broker")
def signal_names(
        url: str = typer.Option(..., help="Broker URL", envvar='REMOTIVE_BROKER_URL'),
        api_key: str = typer.Option("offline", help="Cloud Broker API-KEY or access token",
                                    envvar='REMOTIVE_BROKER_API_KEY')
):
    broker = Broker(url, api_key)
    # print("Listing available signals")
    available_signals = broker.list_signal_names()
    print(json.dumps(available_signals))


@app.command(help="Subscribe to signals")
def subscribe(
        url: str = typer.Option(..., help="Broker URL", envvar='REMOTIVE_BROKER_URL'),
        api_key: str = typer.Option("offline", help="Cloud Broker API-KEY or access token",
                                    envvar='REMOTIVE_BROKER_API_KEY'),
        signal: List[str] = typer.Option(..., help="Roles to apply"),
        namespace: str = typer.Option(..., help="Cloud Broker API-KEY or access token",
                                      envvar='REMOTIVE_BROKER_API_KEY'),
        on_change_only: bool = typer.Option(default=False, help="Only get signal if value is changed"),
        # samples: int = typer.Option(default=0, he)

):
    broker = Broker(url, api_key)
    print("Subscribing to signals, press Ctrl+C to exit")

    def exit_on_ctrlc(sig, frame):
        os._exit(0)

    def on_frame_func(x):
        rich_rprint(json.dumps(list(x)))

    os_signal.signal(os_signal.SIGINT, exit_on_ctrlc)

    #print(namespace)
    #signals2 = list(map( lambda s: s['signal'], broker.list_signal_names2(namespace)))

    broker.subscribe(signals, namespace, on_frame_func, on_change_only)


@app.command(help="List namespaces on broker")
def namespaces(
        url: str = typer.Option(..., help="Broker URL", envvar='REMOTIVE_BROKER_URL'),
        api_key: str = typer.Option("offline", help="Cloud Broker API-KEY or access token",
                                    envvar='REMOTIVE_BROKER_API_KEY')
):
    broker = Broker(url, api_key)
    namespaces_json = broker.list_namespaces()
    print(json.dumps(namespaces_json))



import json
from pathlib import Path
import os

import typer

from . import rest_helper as rest

app = typer.Typer()

config_dir_name = str(Path.home()) + "/.config/.remotive/"
token_file_name = str(Path.home()) + "/.config/.remotive/cloud.secret.token2"


@app.command(name="create", help="Create new access token")
def create(
        expire_in_days: int = typer.Option(default=365, help="Number of this token is valid"),
        service_account: str = typer.Option(..., help="Service account name"),
        project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):

    response = rest.handle_post(url=f"/api/project/{project}/admin/accounts/{service_account}/keys",
                                return_response=True, body=json.dumps({"daysUntilExpiry": expire_in_days}))

    if response.status_code == 200:
        name = response.json()['name']
        write_token(f"service-account-{service_account}-{name}-token.json", response.text)
    else:
        print(f'Got status code: {response.status_code}')
        print(response.text)


@app.command(name="list", help="List service-account access tokens")
def list_keys(
        service_account: str = typer.Option(..., help="Service account name"),
        project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    rest.handle_get(f"/api/project/{project}/admin/accounts/{service_account}/keys")


@app.command(name="list-files")
def list_files():
    """
    List personal access token files in remotivelabs config directory
    """
    personal_files = filter(lambda f: f.startswith('service-account'), os.listdir(config_dir_name) )
    for file in personal_files:
        print(file)

@app.command(name="revoke", help="Revoke service account access token")
def revoke(
        name: str = typer.Argument(..., help="Access token name"),
        service_account: str = typer.Option(..., help="Service account name"),
        project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    rest.handle_delete(f"/api/project/{project}/admin/accounts/{service_account}/keys/{name}")


def write_token(file, token):
    path = str(Path.home()) + f"/.config/.remotive/{file}"
    f = open(path, "w")
    f.write(token)
    f.close()
    print(f"Secret token written to {path}")



import sys
import signal
import re
import pathlib
from multiprocessing import Process
import typer
import requests
import os

import glob
import json
import shutil
from rich.progress import Progress, SpinnerColumn, TextColumn,TaskProgressColumn
from . import rest_helper as rest

app = typer.Typer()


def uid(p):
    print(p)
    return p['uid']

# to be used in options
#autocompletion=project_names)
def project_names():
    r = requests.get(f'{rest.base_url}/api/bu/{rest.org}/project', headers=rest.headers)
    # sys.stderr.write(r.text)
    if r.status_code == 200:
        projects = r.json()
        names = map(lambda p: p['uid'], projects)
        return (list(names))
    else:
        sys.stderr.write(f"Could not list projects due to {r.status_code}\n")
        # os.kill(signal.SIGSTOP)
        raise typer.Exit(0)
        # return []

        # return map(list(r.json()), lambda e: e.uid)


#    return ["beamyhack"]


@app.command("list")
def listRecordings(is_processing: bool = typer.Option(default=False, help="Use this option to see only those that are beeing processed or are invalid"),
                   project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    """
    List all recording sessions in a project. You can choose to see all valid recordings (default) or use
    --is-processing and you will get those that are currently beeing processed or that failed to be validated.

    """

    if is_processing:
        rest.handle_get(f"/api/project/{project}/files/recording/processing")
    else:
        rest.handle_get(f"/api/project/{project}/files/recording")



@app.command(help="Shows details about a specific recording in project")
def describe(recording_session: str = typer.Argument(..., help="Recording session id"),

             project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    rest.handle_get(f"/api/project/{project}/files/recording/{recording_session}")


def doStart(name: str, project: str, api_key: str, return_response: bool = False):
    if api_key == "":
        body = {"size": "S"}
    else:
        body = {"size": "S", 'apiKey': api_key}
    return rest.handle_post(f"/api/project/{project}/brokers/{name}", body=json.dumps(body),
                            return_response=return_response)


@app.command(help="Plays all recording files or a single recording")
def play(recording_session: str = typer.Argument(..., help="Recording session id"),
         broker: str = typer.Option(..., help="Broker name to play on"),
         ensure_broker_started: bool = typer.Option(default=False, help="Ensure broker exists, start otherwise"),
         broker_config_name: str = typer.Option("default", help="Specify a custom broker configuration to use" ),
         project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
    ) as progress:
        rest.ensure_auth_token()
        v = progress.add_task(description=f"Verifying broker {broker} exists...", total=100)

        r = requests.get(f'{rest.base_url}/api/project/{project}/brokers/{broker}', headers=rest.headers, )
        progress.update(v, advance=100.0)
        if r.status_code == 404:
            if ensure_broker_started:
                progress.add_task(description=f"Starting broker {broker}...", total=1)
                r = doStart(broker, project, '', return_response=True)
                if r.status_code != 200:
                    print(r.text)
                    exit(0)
            else:
                print("Broker not running, use --ensure-broker-started to start the broker")
                exit(0)
        elif r.status_code != 200:
            sys.stderr.write(f"Got http status code {r.status_code}")
            typer.Exit(0)

        progress.add_task(
            description=f"Uploading recording {recording_session} to {broker} and setting play mode to pause...",
            total=None)
        # if recording_file == "":
        #    rest.handle_get(f"/api/project/{project}/files/recording/{recording_session}/upload",
        #                    params={'brokerName': broker})
        # else:
        broker_config_query=""
        if (broker_config_name != "default"):
            broker_config_query = f"?brokerConfigName={broker_config_name}"

        rest.handle_get(f"/api/project/{project}/files/recording/{recording_session}/upload{broker_config_query}",
                        params={'brokerName': broker})


@app.command(help="Downloads the specified recording file to disk")
def download_recording_file(
        recording_file_name: str = typer.Argument(..., help="Recording file to download"),
        recording_session: str = typer.Option(..., help="Recording session id that this file belongs to", envvar='REMOTIVE_CLOUD_RECORDING_SESSION'),
        project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
    ) as progress:
        progress.add_task(description=f"Downloading {recording_file_name}", total=None)

        # First request the download url from cloud. This is a public signed url that is valid
        # for a short period of time
        rest.ensure_auth_token()
        get_signed_url_resp = requests.get(
            f'{rest.base_url}/api/project/{project}/files/recording/{recording_session}/recording-file/{recording_file_name}',
            headers=rest.headers, allow_redirects=True)
        if get_signed_url_resp.status_code == 200:

            # Next download the actual file
            download_resp = requests.get(url=get_signed_url_resp.json()["downloadUrl"], stream=True)
            if download_resp.status_code == 200:
                with open(recording_file_name, 'wb') as out_file:
                    shutil.copyfileobj(download_resp.raw, out_file)
                print(f"{recording_file_name} downloaded")
            else:
                sys.stderr.write(download_resp.text)
                sys.stderr.write(f"Got http status {download_resp.status_code}\n")
        else:
            sys.stderr.write(get_signed_url_resp.text)
            sys.stderr.write(f"Got http dd status {get_signed_url_resp.status_code}\n")


@app.command(name="delete")
def delete(recording_session: str = typer.Argument(..., help="Recording session id"),
           project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    """
    Deletes the specified recording session including all media files and configurations.

    """
    rest.handle_delete(f"/api/project/{project}/files/recording/{recording_session}")


@app.command(name="delete-recording-file")
def delete_recording_file(recording_file_name: str = typer.Argument(..., help="Recording file to download"),
           recording_session: str = typer.Option(..., help="Recording session id that this file belongs to", envvar='REMOTIVE_CLOUD_RECORDING_SESSION'),
           project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    """
    Deletes the specified recording file

    """
    rest.handle_delete(f'/api/project/{project}/files/recording/{recording_session}/recording-file/{recording_file_name}')

@app.command()
def upload(path: str  = typer.Argument(..., help="Path to valid recording to upload"),
           project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
    ) as progress:
        progress.add_task(description=f"Uploading {file}", total=None)

        filename = os.path.basename(file)
        rest.ensure_auth_token()
        rest.headers["content-type"] = "application/octet-stream"
        r = requests.post(f"{rest.base_url}/api/project/{project}/files/recording/{filename}", headers=rest.headers)
        if r.status_code == 200:
            headers = {}
            headers["content-type"] = "application/x-www-form-urlencoded"
            r = requests.put(r.text, open(file, 'rb'), headers=headers)
            print(
                "File successfully uploaded, please run 'remotive cloud recordings list' to verify that the recording was successfully processed")
        else:
            print(r.text)

@app.command()
def upload_broker_configuration(
        directory:str = typer.Argument(..., help="Configuration directory"),
        recording_session: str = typer.Option(..., help="Recording session id", envvar='REMOTIVE_CLOUD_RECORDING_SESSION'),
        project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT'),
        overwrite: bool = typer.Option(False, help="Overwrite existing configuration if it exists")
):
    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
    ) as progress:

        #
        # List files in specified directory. Look for interfaces.json and use that directory where this is located
        # as configuration home directory
        #
        files = list(filter(lambda item : "interfaces.json" in item, glob.iglob(directory + '**/**', recursive=True)))
        if len(files) == 0:
            sys.stderr.write("No interfaces.json found in directory, this file is required")
            raise typer.Exit(1)
        if len(files) > 1:
            sys.stderr.write(f"{len(files)} interfaces.json found in directoryw which is not supported")
            raise typer.Exit(1)
        broker_config_dir_name = os.path.dirname(files[0]).rsplit('/', 1)[-1]

        #
        # Get the current details about broker configurations to see if a config with this
        # name already exists
        #
        task=progress.add_task(description=f"Preparing upload of {broker_config_dir_name}", total=1)
        details_resp=rest.handle_get(f"/api/project/{project}/files/recording/{recording_session}", return_response=True)
        details=details_resp.json()
        existing_configs = details['brokerConfigurations']
        if (len(existing_configs) > 0):
            data=list(filter(lambda x : x['name'] == broker_config_dir_name, existing_configs))
            if len(data) > 0:
                if (overwrite):
                    rest.handle_delete(f'/api/project/{project}/files/recording/{recording_session}/configuration/{broker_config_dir_name}', quiet=True)
                else:
                    sys.stderr.write("Broker configuration already exists, use --overwrite to replace\n")
                    raise typer.Exit(1)

        #
        # From the list of files, create a tuple of local_path to the actual file
        # and a remote path as it should be stored in cloud
        #
        file_infos = list(map(lambda item :
                     { 'local_path' : item,
                     'remote_path' : f"/{broker_config_dir_name}{item.rsplit(broker_config_dir_name,1)[-1]}"} ,glob.iglob(directory + '**/*.*', recursive=True)))

        #
        # convert this remote paths and ask cloud to prepare upload urls for those
        #
        json_request_upload_urls_req = {
            'name' : 'not_used',
            'paths' : list(map(lambda x: x['remote_path'], file_infos))
        }

        response = rest.handle_put(url = f'/api/project/{project}/files/recording/{recording_session}/configuration', return_response=True, body=json.dumps(json_request_upload_urls_req))
        if (response.status_code != 200):
            print(f'Failed to prepare configuration upload')
            print(f'{response.text} - {response.status_code}')
            raise typer.Exit(1)
        progress.update(task, advance=1)



        task=progress.add_task(description=f"Uploading {broker_config_dir_name}", total=len(file_infos))

        #
        # Upload urls is a  remote_path : upload_url dict
        # '/my_config/interfaces.json' : "<upload_url>"
        #
        upload_urls = json.loads(response.text)

        # For each file - upload
        for file in file_infos:
            key=file['remote_path']
            path=file['local_path']
            url=upload_urls[key]
            headers={
                'Content-Type' : "application/x-www-form-urlencoded"
            }
            r = requests.put(url, open(path, 'rb'), headers=headers)
            progress.update(task, advance=1)
            if r.status_code != 200:
                print("Failed to upload broker configuration")
                print(r.status_code)
                print(r.text)
                typer.Exit(1)

        print(f"Successfully uploaded broker configuration {broker_config_dir_name}")



@app.command(help="Downloads the specified broker configuration directory as zip file")
def download_configuration(
        broker_config_name: str = typer.Argument(..., help="Broker config name"),
        recording_session: str = typer.Option(..., help="Recording session id", envvar='REMOTIVE_CLOUD_RECORDING_SESSION'),
        project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')
):
    rest.ensure_auth_token()
    # print(rest.base_url)
    r = rest.handle_get(
        url=f"/api/project/{project}/files/recording/{recording_session}/configuration/{broker_config_name}",
        return_response=True)
    # print(r.status_code)
    # print(r.headers)
    # print(get_filename_from_cd(r.headers.get('content-disposition')))
    filename = get_filename_from_cd(r.headers.get('content-disposition'))
    open(filename, 'wb').write(r.content)
    print(f'Downloaded file {filename}')

@app.command(help="Downloads the specified broker configuration directory as zip file")
def delete_configuration(
        broker_config_name: str = typer.Argument(..., help="Broker config name"),
        recording_session: str = typer.Option(..., help="Recording session id", envvar='REMOTIVE_CLOUD_RECORDING_SESSION'),
        project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')
):

    rest.handle_delete(
        url=f"/api/project/{project}/files/recording/{recording_session}/configuration/{broker_config_name}")



def get_filename_from_cd(cd):
    """
    Get filename from content-disposition
    """
    if not cd:
        return None
    fname = re.findall('filename=(.+)', cd)
    if len(fname) == 0:
        return None
    return fname[0]



import shutil
import sys

import requests
import typer
from rich.progress import Progress, SpinnerColumn, TextColumn

from . import rest_helper as rest

app = typer.Typer()


@app.command("list")
def list_signal_databases(project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    rest.handle_get(f'/api/project/{project}/files/config')


@app.command("delete")
def delete(signal_db_file: str = typer.Argument("", help="Signal database file"),
           project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    rest.handle_delete(f'/api/project/{project}/files/config/{signal_db_file}')


@app.command()
def download(signal_db_file: str = typer.Argument("", help="Signal database file"),
             project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
    ) as progress:

        rest.ensure_auth_token()

        progress.add_task(description=f"Downloading {signal_db_file}", total=None)

        # First request the download url from cloud. This is a public signed url that is valid
        # for a short period of time
        get_signed_url_resp = requests.get(
            f'{rest.base_url}/api/project/{project}/files/config/{signal_db_file}/download',
            headers=rest.headers, allow_redirects=True)
        if get_signed_url_resp.status_code == 200:

            # Next download the actual file
            download_resp = requests.get(url=get_signed_url_resp.text, stream=True)
            if download_resp.status_code == 200:
                with open(signal_db_file, 'wb') as out_file:
                    shutil.copyfileobj(download_resp.raw, out_file)
                print(f"{signal_db_file} downloaded")
            else:
                sys.stderr.write(f"Got unexpected status {download_resp.status_code}\n")
        else:
            sys.stderr.write(f"Got unexpected status {get_signed_url_resp.status_code}\n")

# @app.command()
# def upload(file: str, project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
# files = {'upload_file': open(file, 'rb')}
# values = {'DB': 'photcat', 'OUT': 'csv', 'SHORT': 'short'}
# rest.headers["content-type"] = "application/octet-stream"
# r = requests.get(f"{rest.base_url}/api/project/{project}/files/recording/upload/{file}", headers=rest.headers)
# print(r.status_code)
# print(r.text)

# curl -X PUT -H 'Content-Type: application/octet-stream' --upload-file docker-compose.yml 'https://storage.googleapis.com/beamylabs-fileuploads-dev/projects/beamyhack/recording/myrecording?X-Goog-Algorithm=GOOG4-RSA-SHA256&X-Goog-Credential=recordings-upload-account%40beamycloud-dev.iam.gserviceaccount.com%2F20220729%2Fauto%2Fstorage%2Fgoog4_request&X-Goog-Date=20220729T134012Z&X-Goog-Expires=3000&X-Goog-SignedHeaders=content-type%3Bhost&X-Goog-Signature=d1fa7639349d6453aebfce8814d6e5685af03952d07aa4e3cb0d44dba7cf5e572f684c8120dba17cbc7ea6a0ef5450542a3c745c65e04272b34265d0ddcf1b67e6f2b5bfa446264a62d77bd7faabf45ad6bd2aec5225f57004b0a31cfe0480cba063a3807d86346b1da99ecbae3f3e6da8f44f06396dfc1fdc6f89e475abdf969142cef6f369f03aff41000c8abb28aa82185246746fd6c16b6b381baa2d586382a3d3067b6376ddba2b55b2b6f9d942913a1cbfbc61491ba6a615d7d5a0d9a476c357431143e9cea1411dfad9f01b1e1176dc8c056cbf08cccfd401a55d63c19d038f3ab42b712abc48d759047ac07862c4fae937c341e19b568bb60a4e4086'



import json
from typing import List

import typer

from . import rest_helper as rest
from . import service_account_tokens

app = typer.Typer()


@app.command(name="list", help="List service-accounts")
def list_service_accounts(project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    rest.handle_get(f"/api/project/{project}/admin/accounts")


@app.command(name="create", help="Create service account")
def create_service_account(
        name: str,
        role: List[str] = typer.Option(..., help="Roles to apply"),
        project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    data = {
        'name': name,
        'roles': role
    }
    rest.handle_post(url=f"/api/project/{project}/admin/accounts", body=json.dumps(data))


@app.command(name="update", help="Update service account")
def update_service_account(
        service_account: str = typer.Option(..., help="Service account name"),
        role: List[str] = typer.Option(..., help="Roles to apply"),
        project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):

    rest.handle_put(url=f"/api/project/{project}/admin/accounts/{service_account}", body=json.dumps(role))


@app.command(name="delete", help="Delete service account")
def delete_service_account(
        name: str,
        project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    rest.handle_delete(url=f"/api/project/{project}/admin/accounts/{name}")


app.add_typer(service_account_tokens.app, name="tokens", help="Manage project service account access tokens")



import typer
import websocket
import json
import signal as os_signal
import os
from rich.progress import Progress, SpinnerColumn, TextColumn
from . import rest_helper as rest
#from rich import print

app = typer.Typer()


@app.command("list", help="Lists brokers in project")
def list(project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    rest.handle_get(f"/api/project/{project}/brokers")


@app.command("describe", help="Shows details about a specific broker")
def describe(name: str, project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    rest.handle_get(f"/api/project/{project}/brokers/{name}")


@app.command("remove", help="Stops and removes a broker from project")
def stop(name: str, project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
    ) as progress:
        progress.add_task(description=f"Stopping broker {name}...", total=None)
        rest.handle_delete(f"/api/project/{project}/brokers/{name}")


def do_start(name: str, project: str, api_key: str, tag: str, return_response: bool = False):
    if tag == "":
        tag_to_use = None
    else:
        tag_to_use = tag

    if api_key == "":
        body = {"size": "S", 'tag': tag_to_use}
    else:
        body = {"size": "S", 'apiKey': api_key, 'tag': tag_to_use}
    return rest.handle_post(f"/api/project/{project}/brokers/{name}", body=json.dumps(body),
                            return_response=return_response)


@app.command(name="create", help="Create a broker in project")
def start(name: str,
          project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT'),
          tag: str = typer.Option("", help="Optional specific tag/version"),
          silent: bool = typer.Option(False, help="Optional specific tag/version"),
          api_key: str = typer.Option("", help="Start with your own api-key")):
    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
    ) as progress:
        progress.add_task(description=f"Starting broker {name}...", total=None)
        do_start(name, project, api_key, tag, return_response=silent)


@app.command(name="logs")
def logs(
        broker_name: str = typer.Argument(...,help="Broker name to see logs for"),
        tail: bool = typer.Option(False, help="Tail the broker log"),
        history_minutes: str = typer.Option(10, help="History in minutes minutes to fetch"),
        project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):

    """
    Exposes broker logs history or real-time tail of the broker.

    When using --tail option, --history always skipped even if supplied

    """

    def exit_on_ctrlc(sig, frame):
        wsapp.close()
        os._exit(0)

    os_signal.signal(os_signal.SIGINT, exit_on_ctrlc)

    def on_message(wsapp, message):
        print(message)

    def on_error(wsapp, err):
        print("EXAMPLE error encountered: ", err)

    rest.ensure_auth_token()
    # This will work with both http -> ws and https -> wss
    ws_url = rest.base_url.replace('http', 'ws')

    if tail:
        q = "?tail=yes"
    elif history_minutes != 10:
        q = f"?history={history_minutes}"
    else:
        q = ""

    wsapp = websocket.WebSocketApp(f"{ws_url}/api/project/{project}/brokers/{broker_name}/logs{q}", header=rest.headers,
                                   on_message=on_message, on_error=on_error)
    wsapp.run_forever()



import typer
from typing import List
from .lib.broker import Broker

app = typer.Typer(help=help)

@app.command()
def start(
        filename: str = typer.Argument(..., help="Path to local file to upload"),
        namespace: List[str] = typer.Option(..., help="Namespace to record"),
        url: str = typer.Option(..., help="Broker URL", envvar='REMOTIVE_BROKER_URL'),
        api_key: str = typer.Option("offline", help="Cloud Broker API-KEY or access token",
                                    envvar='REMOTIVE_BROKER_API_KEY')
):
    broker = Broker(url, api_key)
    broker.record_multiple(namespace, filename)


@app.command()
def stop(
        filename: str = typer.Argument(..., help="Path to local file to upload"),
        namespace: List[str] = typer.Option(..., help="Namespace to record"),
        url: str = typer.Option(..., help="Broker URL", envvar='REMOTIVE_BROKER_URL'),
        api_key: str = typer.Option("offline", help="Cloud Broker API-KEY or access token",
                                    envvar='REMOTIVE_BROKER_API_KEY')
):
    broker = Broker(url, api_key)
    broker.stop_multiple(namespace, filename)
    print("Successfully stopped recording")


import typer
import base64
from .broker.brokers import app as broker_app
from .cloud.cloud_cli import app as cloud_app
from rich import print

app = typer.Typer(help="""
Welcome to RemotiveLabs CLI - Simplify and automate tasks for cloud resources and brokers
""")


app.add_typer(broker_app, name="broker", help="Manage a single broker - local or cloud")
app.add_typer(cloud_app, name="cloud", help="Manage resources in RemotiveCloud", )



import json

import typer

from . import recordings, brokers, configs, auth, service_accounts, projects
from . import rest_helper as rest

app = typer.Typer()


@app.command(help="List licenses for an organisation")
def licenses(
        organisation: str = typer.Option(..., help="Organisation ID", envvar='REMOTIVE_CLOUD_ORGANISATION'),
        filter: str = typer.Option("all", help="all, valid, expired")
):
    rest.handle_get(f"/api/bu/{organisation}/licenses", {'filter': filter})


@app.command(help="List your available organisations")
def organisations():
    r = rest.handle_get(f"/api/home", return_response=True)
    if r.status_code == 200:
        j = list(map(lambda x : x["billableUnitUser"]["billableUnit"]["uid"], r.json()))
        print(json.dumps(j))
    else:
        print(f"Got status code: {r.status_code}")
        print(r.text)


app.add_typer(projects.app, name="projects", help="Manage projects")
app.add_typer(auth.app, name="auth")
app.add_typer(brokers.app, name="brokers", help="Manage cloud broker lifecycle")
app.add_typer(recordings.app, name="recordings", help="Manage recordings")
app.add_typer(configs.app, name="signal-databases", help="Manage signal databases")
app.add_typer(service_accounts.app, name="service-accounts", help="Manage project service account keys")


if __name__ == "__main__":
    app()



import typer
from typing import List
from .lib.broker import Broker

app = typer.Typer(help=help)


@app.command()
def play(recording: List[str] = typer.Option(..., help="Which recording and which namespace to play"),
         url: str = typer.Option(..., help="Broker URL", envvar='REMOTIVE_BROKER_URL'),
         api_key: str = typer.Option("offline", help="Cloud Broker API-KEY or access token",
                                     envvar='REMOTIVE_BROKER_API_KEY')):
    """
    Play recording files on broker.

    Separate recording file and namespace with ::

    remotive broker playback play --recording myrecording_can0::can0 --recording myrecording_can1::can1
    """

    def recording_and_namespace(recording: str):
        splitted = recording.split('::')
        if len(splitted) != 2:
            print("Invalid --recording option, expected file_name::namespace")
            raise typer.Exit(1)
        return {
            'recording': splitted[0],
            'namespace': splitted[1]
        }

    rec=list(map(recording_and_namespace, recording))

    broker = Broker(url, api_key)
    broker.play(rec)


@app.command()
def stop(
        recording: List[str] = typer.Option(..., help="Which recording and which namespace to stop"),
        url: str = typer.Option(..., help="Broker URL", envvar='REMOTIVE_BROKER_URL'),
        api_key: str = typer.Option("offline", help="Cloud Broker API-KEY or access token",
                                    envvar='REMOTIVE_BROKER_API_KEY')
):
    """
   Stop recordings that are beeing played on brokers are done with the same syntax as when you start them.

   Separate recording file and namespace with ::

   remotive broker playback stop --recording myrecording_can0::can0 --recording myrecording_can1::can1
   """

    broker = Broker(url, api_key)
    def recording_and_namespace(recording: str):
        splitted = recording.split('::')
        if len(splitted) != 2:
            print("Invalid --recording option, expected file_name::namespace")
            raise typer.Exit(1)
        return {
            'recording': splitted[0],
            'namespace': splitted[1]
        }

    rec=list(map(recording_and_namespace, recording))

    broker = Broker(url, api_key)
    broker.stop_play(rec)
    print("Successfully stopped recording")


@app.command()
def pause(
        recording: List[str] = typer.Option(..., help="Which recording and which namespace to stop"),
        url: str = typer.Option(..., help="Broker URL", envvar='REMOTIVE_BROKER_URL'),
        api_key: str = typer.Option("offline", help="Cloud Broker API-KEY or access token",
                                    envvar='REMOTIVE_BROKER_API_KEY')
):
    """
    Pause recordings that are beeing played on brokers are done with the same syntax as when you start them.

    Separate recording file and namespace with ::

    remotive broker playback pause --recording myrecording_can0::can0 --recording myrecording_can1::can1
    """

    broker = Broker(url, api_key)
    def recording_and_namespace(recording: str):
        splitted = recording.split('::')
        if len(splitted) != 2:
            print("Invalid --recording option, expected file_name::namespace")
            raise typer.Exit(1)
        return {
            'recording': splitted[0],
            'namespace': splitted[1]
        }

    rec=list(map(recording_and_namespace, recording))

    broker = Broker(url, api_key)
    broker.pause_play(rec)
    print("Successfully stopped recording")



from pathlib import Path
import os
import sys
import json
import typer
from . import rest_helper as rest

app = typer.Typer()

token_file_name = str(Path.home()) + "/.config/.remotive/cloud.secret.token"
config_dir_name = str(Path.home()) + "/.config/.remotive/"


@app.command(name="create", help="Create and download a new personal access token")
def get_personal_access_token():
    rest.ensure_auth_token()
    rest.handle_post('/api/me/keys')

    response = rest.handle_post(url='/api/me/keys',
                                return_response=True)

    if response.status_code == 200:
        name = response.json()['name']
        write_personal_token(f"personal-token-{name}.json", response.text)
    else:
        print(f'Got status code: {response.status_code}')
        print(response.text)


@app.command(name="list", help="List personal access tokens")
def list_personal_access_tokens():
    rest.ensure_auth_token()
    rest.handle_get('/api/me/keys')


@app.command(name="revoke", help="Revoke the specified access token")
def revoke(
        name: str = typer.Option(..., help="Name of the access token to revoke")
):
    rest.ensure_auth_token()
    rest.handle_delete(f'/api/me/keys/{name}', success_msg="Successfully revoked")

@app.command()
def describe(
        file: str = typer.Option(..., help="File name")):
    """
    Show contents of specified access token file
    """
    print(read_file(file))


@app.command()
def activate(
        file: str = typer.Argument(..., help="File name")):
    """
    Activate a access token file to be used for authentication.

    --file

    This will be used as the current access token in all subsequent requests. This would
    be the same as login with a browser.
    """
    # Best effort to read file
    if os.path.exists(file):
        token_file = json.loads(read_file_with_path(file))
        write_token(token_file['token'])
    elif os.path.exists(str(Path.home()) + f"/.config/.remotive/{file}"):
        token_file = json.loads(read_file(file))
        write_token(token_file['token'])
    else:
        sys.stderr.write("File could not be found \n")


@app.command(name="list-files")
def list_files():
    """
    List personal access token files in remotivelabs config directory
    """
    personal_files = filter(lambda f: f.startswith('personal'), os.listdir(config_dir_name) )
    for file in personal_files:
        print(file)


def read_file(file):
    f = open(str(Path.home()) + f"/.config/.remotive/{file}", "r")
    token = f.read()
    f.close()
    return token


def read_file_with_path(file):
    f = open(file, "r")
    token = f.read()
    f.close()
    return token


def write_token(token):
    f = open(token_file_name, "w")
    f.write(token)
    f.close()


def write_personal_token(file, token):
    path = str(Path.home()) + f"/.config/.remotive/{file}"
    f = open(path, "w")
    f.write(token)
    f.close()
    print(f"Personal access token written to {path}")
    print(f"Use 'remotive cloud auth activate {file}' to use this access token from cli")
    print('\033[93m This file contains secrets and must be handled there after')



import signal as os_signal
import os
import signal
import time
import grpc
import ntpath
import posixpath
import binascii
import tempfile
import zipfile
import queue
from threading import Thread
import remotivelabs.broker.sync as br
import remotivelabs.broker.sync.helper as br_helper
import typer
from typing import List
import remotivelabs.broker.generated.sync.traffic_api_pb2 as traffic_api


class Broker:

    def __init__(self, url, api_key):
        self.url = url
        self.api_key = api_key
        self.q = queue.Queue()
        """Main function, checking arguments passed to script, setting up stubs, configuration and starting Threads."""
        # Setting up stubs and configuration
        self.intercept_channel = br.create_channel(url, api_key)

        self.network_stub = br.network_api_pb2_grpc.NetworkServiceStub(self.intercept_channel)
        self.system_stub = br.system_api_pb2_grpc.SystemServiceStub(self.intercept_channel)
        self.traffic_stub = br.traffic_api_pb2_grpc.TrafficServiceStub(self.intercept_channel)
        self.signal_creator = br.SignalCreator(self.system_stub)

    def play(self, recording_and_namespace: List):

        def to_playback(rec):
            return {
                "namespace": rec['namespace'],
                "path": rec['recording'],
                "mode": br.traffic_api_pb2.Mode.PLAY,
            }

        playback_list = map(to_playback, recording_and_namespace)

        status = self.traffic_stub.PlayTraffic(
            br.traffic_api_pb2.PlaybackInfos(
                playbackInfo=list(map(self.__create_playback_config, playback_list))
            )
        )

    def stop_play(self, recording_and_namespace: List):

        def to_playback(rec):
            return {
                "namespace": rec['namespace'],
                "path": rec['recording'],
                "mode": br.traffic_api_pb2.Mode.STOP,
            }

        playback_list = map(to_playback, recording_and_namespace)

        status = self.traffic_stub.PlayTraffic(
            br.traffic_api_pb2.PlaybackInfos(
                playbackInfo=list(map(self.__create_playback_config, playback_list))
            )
        )

    def pause_play(self, recording_and_namespace: List):

        def to_playback(rec):
            return {
                "namespace": rec['namespace'],
                "path": rec['recording'],
                "mode": br.traffic_api_pb2.Mode.PAUSE,
            }

        playback_list = map(to_playback, recording_and_namespace)

        status = self.traffic_stub.PlayTraffic(
            br.traffic_api_pb2.PlaybackInfos(
                playbackInfo=list(map(self.__create_playback_config, playback_list))
            )
        )
            # print(status)

    def record_multiple(self, namespaces: List[str], path: str):

        def to_playback(namespace):
            return {
                "namespace": namespace,
                "path": path + "_" + namespace,
                "mode": br.traffic_api_pb2.Mode.RECORD,
            }

        playback_list = list(map(to_playback, namespaces))

        status = self.traffic_stub.PlayTraffic(
            br.traffic_api_pb2.PlaybackInfos(
                playbackInfo=list(map(self.__create_playback_config, playback_list))
            )
        )

    def record(self, namespace: str, path: str):
        playback_list = [
            {
                "namespace": namespace,
                "path": path,
                "mode": br.traffic_api_pb2.Mode.RECORD,
            }]

        status = self.traffic_stub.PlayTraffic(
            br.traffic_api_pb2.PlaybackInfos(
                playbackInfo=list(map(self.__create_playback_config, playback_list))
            )
        )
        # print(status)

    def stop(self, namespace: str, path: str):
        playback_list = [
            {
                "namespace": namespace,
                "path": path,
                "mode": br.traffic_api_pb2.Mode.STOP,
            }]

        status: traffic_api.PlaybackInfos = self.traffic_stub.PlayTraffic(
            br.traffic_api_pb2.PlaybackInfos(
                playbackInfo=list(map(self.__create_playback_config, playback_list))
            )
        )
        mode = status.playbackInfo[0].playbackMode
        if mode.errorMessage:
            print(f'Error stopping recording {mode.errorMessage}')
            raise typer.Exit(1)

    def pause(self, namespace: str, path: str):
        playback_list = [
            {
                "namespace": namespace,
                "path": path,
                "mode": br.traffic_api_pb2.Mode.PAUSE,
            }]

        status: traffic_api.PlaybackInfos = self.traffic_stub.PlayTraffic(
            br.traffic_api_pb2.PlaybackInfos(
                playbackInfo=list(map(self.__create_playback_config, playback_list))
            )
        )
        mode = status.playbackInfo[0].playbackMode
        if mode.errorMessage:
            print(f'Error pausing recording {mode.errorMessage}')
            raise typer.Exit(1)

    def stop_multiple(self, namespaces: List[str], path: str):
        def to_playback(namespace):
            return {
                "namespace": namespace,
                "path": path + "_" + namespace,
                "mode": br.traffic_api_pb2.Mode.STOP,
            }

        playback_list = list(map(to_playback, namespaces))

        status = self.traffic_stub.PlayTraffic(
            br.traffic_api_pb2.PlaybackInfos(
                playbackInfo=list(map(self.__create_playback_config, playback_list))
            )
        )
        mode = status.playbackInfo[0].playbackMode
        if mode.errorMessage:
            print(f'Error stopping recording {mode.errorMessage}')
            raise typer.Exit(1)

    def diagnose_stop(self, namespace: List[str]):
        recording_name = f"diagnose__"
        self.stop_multiple(namespace, recording_name)

    def diagnose(self, namespace: List[str], wait_for_traffic: bool = False):
        recording_name = f"diagnose__"

        keep_running = True
        keep_running_during_recording = True
        def exit_on_ctrlc(sig, frame):
            nonlocal keep_running
            keep_running = False
            nonlocal keep_running_during_recording
            keep_running_during_recording = False
            #progress.add_task(description=f"Cleaning up, please wait...", total=None)

        os_signal.signal(os_signal.SIGINT, exit_on_ctrlc)

        while keep_running:
            keep_running = wait_for_traffic
            self.record_multiple(namespace, recording_name)
            for i in range(5):
                if keep_running_during_recording:
                    time.sleep(1)

            self.stop_multiple(namespace, recording_name)

            response = []
            with tempfile.TemporaryDirectory() as tmpdirname:
                for ns in namespace:
                    path = recording_name + "_" + ns
                    tmp_file = os.path.join(tmpdirname, path)
                    self.download(path, tmp_file)
                    self.delete_files([path], False)
                    with zipfile.ZipFile(tmp_file, 'r') as zip_ref:
                        zip_ref.extractall(tmpdirname)

                        file_stat = os.stat(os.path.join(tmpdirname, path + '.raw'))
                        response.append({'namespace': ns, 'data': file_stat.st_size > 0})

            for r in response:
                if r['data']:
                    print(f'Successfully received traffic on {r["namespace"]}')
                    keep_running = False
                else:
                    if not wait_for_traffic or (not keep_running and not keep_running_during_recording):
                        print(f'Namespace {r["namespace"]} did not receive any traffic')



    def upload(self, file: str, dest: str):
        try:
            br.helper.upload_file(system_stub=self.system_stub, path=file, dest_path=dest)
        except grpc.RpcError as rpc_error:
            print(f"Failed to upload file - {rpc_error.details()} ({rpc_error.code()})")
            raise typer.Exit(1)


    def delete_files(self,
            path: List[str],
                     exitOnFaliure:bool
    ) -> None:

            for file in path:
                try:
                    self.system_stub.BatchDeleteFiles(
                        br.system_api_pb2.FileDescriptions(
                                fileDescriptions=[br.system_api_pb2.FileDescription(path=file.replace(ntpath.sep,
                                                                                                   posixpath.sep))]
                            )
                    )
                except grpc.RpcError as rpc_error:
                    print(f"Failed to delete file - {rpc_error.details()} ({rpc_error.code()})")
                    if exitOnFaliure:
                        raise typer.Exit(1)

        #rpc BatchDeleteFiles (FileDescriptions) returns (Empty) {}

    def upload_folder(self, folder: str):
        try:
            br.helper.upload_folder(system_stub=self.system_stub, folder=folder)
        except grpc.RpcError as rpc_error:
            print(f"Failed to upload file - {rpc_error.details()} ({rpc_error.code()})")
            raise typer.Exit(1)

    def download(self, file: str, dest: str):
        try:
            br_helper.download_file(system_stub=self.system_stub, path=file, dest_path=dest)
        except grpc.RpcError as rpc_error:
            print(f"Failed to download file - {rpc_error.details()} ({rpc_error.code()})")
            # There will be an empty file if the download fails so remove that one here
            os.remove(dest)
            raise typer.Exit(1)

    def reload_config(self):
        try:
            request = br.common_pb2.Empty()
            response = self.system_stub.ReloadConfiguration(request, timeout=60000) 
            if response.errorMessage:
                print(f"Failed to reload config: {response.errorMessage}")
                raise typer.Exit(1)
            #br.helper.reload_configuration(system_stub=self.system_stub)
        except grpc.RpcError as rpc_error:
            print(f"Failed to reload configuration - {rpc_error.details()} ({rpc_error.code()})")
            raise typer.Exit(1)

    def list_namespaces(self):
        # Lists available signals
        configuration = self.system_stub.GetConfiguration(br.common_pb2.Empty())
        namespaces = []
        for networkInfo in configuration.networkInfo:
            namespaces.append(networkInfo.namespace.name)
        return namespaces

    def list_signal_names(self, namespace):
        # Lists available signals
        configuration = self.system_stub.GetConfiguration(br.common_pb2.Empty())

        signal_names = []
        for networkInfo in configuration.networkInfo:
            res = self.system_stub.ListSignals(networkInfo.namespace)
            for finfo in res.frame:
                for sinfo in finfo.childInfo:
                    signal_names.append({'signal': sinfo.id.name, 'namespace': networkInfo.namespace.name})
        return signal_names

    def list_signal_names(self):
        # Lists available signals
        configuration = self.system_stub.GetConfiguration(br.common_pb2.Empty())

        signal_names = []
        for networkInfo in configuration.networkInfo:
            res = self.system_stub.ListSignals(networkInfo.namespace)
            for finfo in res.frame:
                for sinfo in finfo.childInfo:
                    signal_names.append({'signal': sinfo.id.name, 'namespace': networkInfo.namespace.name})
        return signal_names

    def subscribe(self, signals: list, namespace, on_frame, changed_values_only: bool = True):
        client_id = br.common_pb2.ClientId(id="cloud_demo")

        signals_to_subscribe_on = \
            map(lambda signal: self.signal_creator.signal(signal, namespace), signals)

        Thread(
            target=br.act_on_signal,
            args=(
                client_id,
                self.network_stub,
                signals_to_subscribe_on,
                changed_values_only,  # True: only report when signal changes
                lambda frame: self.__each_signal(frame, on_frame),
                lambda sub: (self.q.put(("cloud_demo", sub))),
            ),
        ).start()
        # Wait for subscription
        ecu, subscription = self.q.get()
        return subscription

    @classmethod
    def connect(cls, url, api_key):
        return Broker(url, api_key)

    def __each_signal(self, signals, callback):
        callback(map(lambda s: {
            'timestamp_nanos': s.timestamp,
            'name': s.id.name,
            'value': self.__get_value(s)
        }, signals))

    @staticmethod
    def __get_value(signal):
        if signal.raw != b"":
            return "0x" + binascii.hexlify(signal.raw).decode("ascii")
        elif signal.HasField("integer"):
            return signal.integer
        elif signal.HasField("double"):
            return signal.double
        elif signal.HasField("arbitration"):
            return signal.arbitration
        else:
            return "empty"

    @staticmethod
    def __create_playback_config(item):
        """Creating configuration for playback

        Parameters
        ----------
        item : dict
            Dictionary containing 'path', 'namespace' and 'mode'

        Returns
        -------
        PlaybackInfo
            Object instance of class

        """
        playbackConfig = br.traffic_api_pb2.PlaybackConfig(
            fileDescription=br.system_api_pb2.FileDescription(path=item["path"]),
            namespace=br.common_pb2.NameSpace(name=item["namespace"]),
        )
        return br.traffic_api_pb2.PlaybackInfo(
            playbackConfig=playbackConfig,
            playbackMode=br.traffic_api_pb2.PlaybackMode(mode=item["mode"]),
        )


Broker.connect = classmethod(Broker.connect)



import json
import signal as os_signal
import os
from time import sleep
from typing import List
from rich import print as rich_rprint
from rich.progress import Progress, SpinnerColumn, TextColumn
import grpc
import typer

from zeroconf import (
    IPVersion,
    ServiceBrowser,
    ServiceStateChange,
    Zeroconf,
)

from .lib.broker import Broker
from . import playback, record, files, signals

app = typer.Typer()


@app.command(name="x-diagnose")
def diagnose(
        namespace: List[str] = typer.Option([], help="Namespace to diagnose"),
        all_namespaces: bool = typer.Option(False, help="Scan all namespaces, must be true if no namespaces provided"),
        wait_for_traffic: bool = typer.Option(False, help="Wait until traffic is found on any bus"),
        quiet: bool = typer.Option(False, help="Do not prompt me"),
        url: str = typer.Option(..., help="Broker URL", envvar='REMOTIVE_BROKER_URL'),
        api_key: str = typer.Option("offline", help="Cloud Broker API-KEY or access token",
                                    envvar='REMOTIVE_BROKER_API_KEY')
):

    """
    [Experimental] - Performs a scan on specified buses to see if the is any traffic coming on the buses. This works
    for both recordings and live data

    """

    if not quiet:
        do_it = typer.confirm("""
        This will perform recordings on selected buses, make sure there are no recordings currently running on the broker.
        """)
        if not do_it:
            print("Skipping diagnosis")
            raise typer.Abort()

    broker = Broker(url, api_key)
    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
    ) as progress:

        if len(namespace) > 0 and all_namespaces:
            print("No namespaces should be supplied when all-namespaces is used")
            raise typer.Exit(1)
        if len(namespace) == 0 and not all_namespaces:
            print("No namespaces chosen")
            raise typer.Exit(1)

        if all_namespaces:
            namespace = broker.list_namespaces()

        if wait_for_traffic:
            progress.add_task(description=f"Scanning for traffic on  {namespace} [Ctrl+C to exit] ", total=1)
        else:
            progress.add_task(description=f"Scanning for traffic on  {namespace}... (just a few seconds)", total=1)

        def on_sigint(sig, frame):
            progress.add_task(description=f"Cleaning up, please wait...", total=None)

        os_signal.signal(os_signal.SIGINT,on_sigint)

        broker.diagnose(namespace, wait_for_traffic)




@app.command(help="Discover brokers on this network")
def discover():
    # print("Not implemented")

    zeroconf = Zeroconf(ip_version=IPVersion.V4Only)

    services = ["_remotivebroker._tcp.local."]
    # services = list(ZeroconfServiceTypes.find(zc=zeroconf))

    print("\nLooking for RemotiveBrokers on your network, press Ctrl-C to exit...\n")
    browser = ServiceBrowser(zeroconf, services, handlers=[on_service_state_change])

    try:
        while True:
            sleep(0.1)
    except KeyboardInterrupt:
        pass
    finally:
        zeroconf.close()


def on_service_state_change(
        zeroconf: Zeroconf, service_type: str, name: str, state_change: ServiceStateChange
) -> None:
    # print(f"Service {name} state changed: {state_change}")

    if state_change is ServiceStateChange.Removed:
        print(f"Service {name} was removed")

    if state_change is ServiceStateChange.Updated:
        print(f"Service {name} was updated")

    if state_change is ServiceStateChange.Added:
        print(f"[ {name} ]")
        info = zeroconf.get_service_info(service_type, name)
        # print("Info from zeroconf.get_service_info: %r" % (info))

        if info:
            # addresses = ["%s:%d" % (addr, cast(int, info.port)) for addr in info.parsed_scoped_addresses()]
            for addr in info.parsed_scoped_addresses():
                print(f'RemotiveBrokerApp: http://{addr}:8080')
                print(f'RemotiveBroker http://{addr}:50051')
            # print("  Weight: %d, priority: %d" % (info.weight, info.priority))
            # print(f"  Server: {info.server}")
            # if info.properties:
            #    print("  Properties are:")
            #    for key, value in info.properties.items():
            #        print(f"    {key}: {value}")
            # else:
            #    print("  No properties")
        else:
            print("  No info")
        print('\n')


app.add_typer(playback.app, name="playback", help="Manage playing recordings")
app.add_typer(record.app, name="record", help="Record data on buses")
app.add_typer(files.app, name="files", help="Upload/Download configurations and recordings")
app.add_typer(signals.app, name="signals", help="Find and subscribe to signals")

if __name__ == "__main__":
    app()



import os

import typer
from typing import List
from rich.progress import Progress, SpinnerColumn, TextColumn

from .lib.broker import Broker

app = typer.Typer(help=help)


@app.command()
def reload_configuration(
        url: str = typer.Option(..., help="Broker URL", envvar='REMOTIVE_BROKER_URL'),
        api_key: str = typer.Option("offline", help="Cloud Broker API-KEY or access token",
                                    envvar='REMOTIVE_BROKER_API_KEY')
):
    broker = Broker(url, api_key)
    broker.reload_config()
    print("Configuration successfully reloaded")



@app.command()
def delete(
        path: List[str] = typer.Argument(..., help="Paths to files on broker to delete"),
        exit_on_failure: bool = typer.Option(False, help="Exits if there was a problem deleting a file"),
        url: str = typer.Option(..., help="Broker URL", envvar='REMOTIVE_BROKER_URL'),
        api_key: str = typer.Option("offline", help="Cloud Broker API-KEY or access token",
                                    envvar='REMOTIVE_BROKER_API_KEY')
):

    """
    Deletes the specified files from the broker
    """

    broker = Broker(url, api_key)

    if len(path) == 0:
        print("At least one path must be suppled")
        raise typer.Exit(1)

    broker.delete_files(path, exit_on_failure)

@app.command()
def download(
        path: str = typer.Argument(..., help="Path to file on broker to download"),
        output: str = typer.Option("", help="Optional output file name"),
        url: str = typer.Option(..., help="Broker URL", envvar='REMOTIVE_BROKER_URL'),
        api_key: str = typer.Option("offline", help="Cloud Broker API-KEY or access token",
                                    envvar='REMOTIVE_BROKER_API_KEY')
):
    """
    Downloads a file from a broker
    """

    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
    ) as progress:
        progress.add_task(description=f"Downloading {path}...", total=None)
        broker = Broker(url, api_key)
        output_file = os.path.basename(path)
        if output != "":
            output_file = output
        if os.path.exists(output_file):
            print(f"File already exist {output_file}, please use another output file name")
        else:
            broker.download(path, output_file)
            print(f"Successfully saved {output_file}")


@app.command()
def upload(
        path: str = typer.Argument(..., help="Path to local file to upload"),
        output: str = typer.Option("", help="Optional output path on broker"),
        url: str = typer.Option(..., help="Broker URL", envvar='REMOTIVE_BROKER_URL'),
        api_key: str = typer.Option("offline", help="Cloud Broker API-KEY or access token",
                                    envvar='REMOTIVE_BROKER_API_KEY')
):
    """
    Uploads a file to a broker - physical or in cloud.
    """

    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
    ) as progress:
        progress.add_task(description=f"Uploading {path}...", total=None)

        if path == "." :
            path = "./" ## Does not work otherwise

        if not os.path.exists(path):
            print(f'File {path} does not exist')
            raise typer.Exit(1)

        broker = Broker(url, api_key)

        if os.path.isdir(path):
            broker.upload_folder(path)
            print(f'Successfully uploaded {path}')
        else:
            output_file = os.path.basename(path)
            if output != "":
                output_file = output
            broker.upload(path, output_file)
            print(f'Successfully uploaded {path}')
